from platform import machine
from sys import platform

from lume.config.check_list_or_str_item import check_list_or_str_item


def get_platform():
    if platform == "linux" or platform == "linux2":
        return "linux"
    elif platform == "darwin":
        if machine() == "arm64":
            return "macos-arm"
        return "macos"
    elif platform == "win32":
        return "windows"


def concat_list(list_of_lists_of_commands: list) -> list:
    if all(v is None for v in list_of_lists_of_commands):
        return None

    result = list()
    for list_of_commands in list_of_lists_of_commands:
        if list_of_commands is None:
            list_of_commands = list()
        result += list_of_commands
    return result


def concat(l1, l2):
    if l1 is None and l2 is None:
        return None
    elif l1 is None:
        l1 = []
    elif l2 is None:
        l2 = []
    s = l1 + l2
    return s


def check_os_list_or_str_item(kdict, key, required=False, suffix=""):
    kvalue = kdict.get(key)
    if not kvalue and required:
        raise TypeError(f"StepConfig must contains {key}{suffix} variable")

    if isinstance(kvalue, dict):
        current_platform = get_platform()
        platform_commands = check_list_or_str_item(kvalue, current_platform)
        all_pre_commands = check_list_or_str_item(kvalue, "all-pre")
        all_post_commands = check_list_or_str_item(kvalue, "all-post")
        all_commands = check_list_or_str_item(kvalue, "all")
        value = concat_list(
            [all_pre_commands, platform_commands, all_commands, all_post_commands]
        )
    else:
        value = check_list_or_str_item(kdict, key)

    return value



from subprocess import Popen

from meiga import Error, Result, Success

from lume.src.domain.services.executor_service import (
    DEFAULT_EXECUTOR_LOG_FILENAME,
    ExecutorService,
)
from lume.src.domain.services.logger import INFO, Logger


class DetachPopenExecutorService(ExecutorService):
    def __init__(self, logger: Logger):
        self.logger = logger

    def execute(
        self, command: str, cwd: str, log_filename: str = DEFAULT_EXECUTOR_LOG_FILENAME
    ) -> Result[bool, Error]:

        if not cwd:
            cwd = "."

        log = open(log_filename, "w")
        process = Popen("exec " + command, stdout=log, stderr=log, cwd=cwd, shell=True)

        self.logger.log(INFO, f"Open process (pid={process.pid}) >> {log_filename}")

        return Success(process)



from meiga import Error


class CrendentialsEnvError(Error):
    def __init__(self, env_name):
        self.message = f"Environment variable ({env_name}) is not defined properly"


class BadZipFileError(Error):
    def __init__(self, resource_name):
        self.message = (
            f"Error decompressing ({resource_name}). Please ensure the url exists"
        )


class BlobNotFoundError(Error):
    def __init__(self, url):
        self.message = f"Blob not found ({url})"


class ItemTypeNotSupportedError(Error):
    def __init__(self, current_type, supported_types):
        self.message = (
            f"Item type ({current_type}) is not supported. Try with ({supported_types})"
        )



from subprocess import Popen

from meiga import Error, Result, isSuccess

from lume.src.application.use_cases.messages import get_colored_command_message
from lume.src.domain.services.killer_service import KillerService
from lume.src.domain.services.logger import COMMAND, Logger


class PopenKillerService(KillerService):
    def __init__(self, logger: Logger):
        self.logger = logger

    def execute(self, process) -> Result[bool, Error]:
        if process and isinstance(process, Popen):
            message = get_colored_command_message(
                f"Killing {process.pid}", None, "auto", "teardown_detach"
            )
            self.logger.log(COMMAND, message)
            process.kill()
            message = get_colored_command_message(
                f"Killed {process.pid}", None, "auto", "teardown_detach"
            )
            self.logger.log(COMMAND, message)

        return isSuccess



from meiga import Error, Result, isSuccess

from lume.src.domain.services.executor_service import (
    DEFAULT_EXECUTOR_LOG_FILENAME,
    ExecutorService,
)


class FakeExecutorService(ExecutorService):
    def execute(
        self, command: str, cwd: str, log_filename: str = DEFAULT_EXECUTOR_LOG_FILENAME
    ) -> Result[bool, Error]:
        print("executing")
        return isSuccess



from lume.src.domain.services.logger import Logger


class FakeLogger(Logger):
    def __init__(self):
        self.logging_messages = []

    def log(self, logging_level, message):
        self.logging_messages.append((logging_level, message))

    def get_logging_messages(self):
        return self.logging_messages



from typing import Dict

from pydantic import BaseModel

from lume.config.dependency_config import DependencyConfig


class SetupConfig(BaseModel):
    deps: Dict[str, DependencyConfig]
    output: str = "deps"



from abc import ABCMeta, abstractmethod

from meiga import Result

from lume.config import DependencyConfig
from lume.src.domain.services.logger import Logger


class SetupItem(object):
    __metaclass__ = ABCMeta
    base_path = None

    def __init__(self, base_path: str):
        self.base_path = base_path

    @abstractmethod
    def run(
        self, name: str, dependency_config: DependencyConfig, logger: Logger
    ) -> Result:
        raise NotImplementedError



from meiga import Success

from lume.config import SetupConfig
from lume.src.domain.services.logger import INFO, WARNING, Logger
from lume.src.domain.services.setup_service import SetupService


class FakeSetupService(SetupService):
    def __init__(self, setup_config: SetupConfig, logger: Logger):
        self.setup_config = setup_config
        self.logger = logger

    def execute(self):

        if not self.setup_config:
            self.logger.log(WARNING, "Empty config for setup")
            return Success()

        self.logger.log(INFO, f"setup: output: {self.setup_config.output}")

        for key, value in self.setup_config.deps.items():
            self.logger.log(INFO, f"{key} : {value}")
        return Success()



import os
from typing import Dict

from lume.src.domain.services.logger import ENVAR, ENVAR_WARNING, GLOBAL, Logger
from lume.src.infrastructure.services.logger.colors import Colors


class EnvManager:
    def __init__(self, logger: Logger):
        self.logger = logger

    def set(self, envs: Dict[str, str]) -> None:
        if envs:
            self.logger.log(
                GLOBAL, f"{Colors.OKGREEN}Set Global Environment Variables{Colors.ENDC}"
            )
        for envar, value in envs.items():
            env_original_value = os.environ.get(envar)
            os.environ[envar] = str(value)
            if env_original_value:
                self.logger.log(
                    ENVAR_WARNING,
                    f"env: overwrite {envar}={value} (Original {envar}={env_original_value})",
                )
            else:
                self.logger.log(ENVAR, f"env: set {envar}={value}")

    def unset(self, envs: Dict[str, str]) -> None:
        for envar in envs.keys():
            os.unsetenv(envar)

    def set_step(self, step):
        for envar, value in step.envs.items():
            env_original_value = os.environ.get(envar)
            os.environ[envar] = str(value)
            if env_original_value:
                self.logger.log(
                    ENVAR_WARNING,
                    f"env: overwrite {envar}={value} (Original {envar}={env_original_value})",
                )
            else:
                if envar in step.overwrote_envs:
                    self.logger.log(
                        ENVAR_WARNING,
                        f"env: overwrite {envar}={value} (Also available on shared envs on lume.yml)",
                    )
                else:
                    self.logger.log(ENVAR, f"env: set {envar}={value}")

    def unset_step(self, step):
        if not step:
            return
        for envar in step.envs.keys():
            os.unsetenv(envar)
